
import pygame
from pygame.locals import *

SCR_W=800
SCR_H=600
block_image_size=60
board_area=pygame.Rect((0,0,block_image_size*8,block_image_size*8))
image_path="bmp\\"
none_image=pygame.image.load(image_path+"none.bmp")
black_image=pygame.image.load(image_path+"black.bmp")
white_image=pygame.image.load(image_path+"white.bmp")
none_code=0
black_code=1
white_code=2
wall_code=3
class board:
    def __init__(self,m):
        self.m=m
    def goho(self,pos,code):
        m=self.m
        if m[pos]!=none_code:
            return False
        opcode=opposite_code(code)
        for dire in [-11,-10,-9,-1,1,9,10,11]:
            if m[pos+dire]==opcode:
                npos=pos+dire
                while m[npos]==opcode:
                    npos+=dire
                if m[npos]==code:
                    return True
        else:
            return False

    def passable(self,code):
        for pos in xrange(100):
            if self.goho(pos,code):
                return False
        else:
            return True
    def put(self,pos,code):
        if pos==-1:
            return board(self.m)
        m=self.m
        new_m=list(self.m)
        opcode=opposite_code(code)
        for dire in [-11,-10,-9,-1,1,9,10,11]:
                npos=pos+dire
                while m[npos]==opcode:
                    npos+=dire
                if m[npos]==code:
                    while npos != pos:
                        npos-=dire
                        new_m[npos]=code
        return board(new_m)

class debug_mes:
    def __init__(self):
        self.mes="OTHRO"
        self.font=pygame.font.Font(None,32)
    def image(self):
        return self.font.render(self.mes,False,(255,255,255))

class user_mes:
    def __init__(self):
        self.mes=["BBBBB","ABC123"]
        self.font=pygame.font.Font(None,32)
    def image(self):
        sur=pygame.Surface((200,32*len(self.mes)))
        y=0
        for sentence in self.mes:
            sur.blit(self.font.render(sentence,False,(255,255,255)),(0,y*32))
            y+=1
        return sur

def computer_thinking(board,code):
    for pos in xrange(64):
        if board.goho(pos,code):
            return pos

def computer_thinking2(board,code):
    goholist=[]
    bestscore=-1000000
    bestpos=-1
    opcode=opposite_code(code)
    nc_number=board.m.count(none_code)
    for pos in xrange(100):
        if board.goho(pos,code):
            goholist.append(pos)
    if 18<=nc_number<=60:
        depth=3
    if 13<=nc_number<=17:
        depth=5
    elif nc_number<=12:
        depth=7
    for pos in goholist:
        score=-eval_board2_2(board.put(pos,code),opcode,depth)
        if score>bestscore:
            bestpos=pos
            bestscore=score
    return bestpos

def eval_board2(board,code):
    value=0
    opcode=opposite_code(code)
    m=board.m
    nc_number=m.count(none_code)
    if nc_number>=12:
        for pos in xrange(100):
            if board.goho(pos,code):
                value+=1
            if board.goho(pos,opcode):
                value-=1
        for pos in [11,18,81,88]:
            if m[pos]==code:
                value+=nc_number
            elif m[pos]==opcode:
                value-=nc_number
        return value
    for pos in [11,18,81,88]:
        if m[pos]==code:
            value+=nc_number
        elif m[pos]==opcode:
            value-=nc_number
    return value+m.count(code)-m.count(opcode)

def eval_board2_2(board,code,depth):
    if depth==0:
        return eval_board2(board,code)
    goholist=[]
    opcode=opposite_code(code)
    bestscore=-1000000
    for pos in xrange(100):
        if board.goho(pos,code):
            goholist.append(pos)
    if goholist==[]:
        return -eval_board2_2(board,opcode,depth-1)
    for pos in goholist:
        score=-eval_board2_2(board.put(pos,code),opcode,depth-1)
        if score>bestscore:
            bestscore=score
    return bestscore


def board_to_image(bod):
    m=bod.m
    sur=pygame.Surface((block_image_size*8,block_image_size*8))
    for y in xrange(1,9):
        for x in xrange(1,9):
            if m[x+10*y]==none_code:
                image=none_image
            elif m[x+10*y]==black_code:
                image=black_image
            elif m[x+10*y]==white_code:
                image=white_image
            sur.blit(image,((x-1)*block_image_size,(y-1)*block_image_size))
    return sur

def init_board():
    m=[3,3,3,3,3,3,3,3,3,3,
       3,0,0,0,0,0,0,0,0,3,
       3,0,0,0,0,0,0,0,0,3,
       3,0,0,0,0,0,0,0,0,3,
       3,0,0,0,2,1,0,0,0,3,
       3,0,0,0,1,2,0,0,0,3,
       3,0,0,0,0,0,0,0,0,3,
       3,0,0,0,0,0,0,0,0,3,
       3,0,0,0,0,0,0,0,0,3,
       3,3,3,3,3,3,3,3,3,3,
       ]
    return board(m)

def opposite_code(code):
    if code==black_code:
        return white_code
    else:
        return black_code

def mousepos_to_pos(pos):
    mx,my=pos
    return mx/block_image_size+1,my/block_image_size+1

def main():
    pygame.init()
    screen=pygame.display.set_mode((SCR_W,SCR_H))
    now_board=init_board()
    pygame.display.flip()
    player_code=black_code
    computer_code=white_code
    dmes=debug_mes()
    umes=user_mes()
    state="player"
    while 1:
        if state=="computer":
            pos=computer_thinking2(now_board,computer_code)
            now_board=now_board.put(pos,computer_code)
            state="player"
        for event in pygame.event.get():
            if event.type==QUIT:
                return
            if event.type==MOUSEBUTTONUP:
                posx,posy=mousepos_to_pos(event.pos)
                if board_area.collidepoint(event.pos):
                    pos=posx+10*posy
                    dmes.mes="%s,%s"%(posx,posy)
                    if now_board.goho(pos,player_code):
                        dmes.mes+="--goho!!"
                        now_board=now_board.put(pos,player_code)
                        state="computer"
        if now_board.passable(player_code):
            if not now_board.passable(computer_code):
                state="computer"
        umes.mes=["black: %s"%(now_board.m.count(black_code)),
                  "white: %s"%(now_board.m.count(white_code)),
                  ]
        screen.fill((0,0,0))
        screen.blit(board_to_image(now_board),(0,0))
        screen.blit(dmes.image(),(0,500))
        screen.blit(umes.image(),(500,0))
        pygame.time.wait(10)
        pygame.display.flip()
main()
